@@ -1,10 +1,28 @@
 /*
- *  SLPDALocator.cpp
- *  NSLPlugins
- *
- *  Created by Kevin Arnold on Thu Oct 05 2000.
- *  Copyright (c) 2000 Apple Computer, Inc. All rights reserved.
+ * Copyright (c) 2002 Apple Computer, Inc. All rights reserved.
  *
+ * @APPLE_LICENSE_HEADER_START@
+ * 
+ * This file contains Original Code and/or Modifications of Original Code
+ * as defined in and that are subject to the Apple Public Source License
+ * Version 2.0 (the 'License'). You may not use this file except in
+ * compliance with the License. Please obtain a copy of the License at
+ * http://www.opensource.apple.com/apsl/ and read it before using this
+ * file.
+ * 
+ * The Original Code and all software distributed under the License are
+ * distributed on an 'AS IS' basis, WITHOUT WARRANTY OF ANY KIND, EITHER
+ * EXPRESS OR IMPLIED, AND APPLE HEREBY DISCLAIMS ALL SUCH WARRANTIES,
+ * INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY,
+ * FITNESS FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT OR NON-INFRINGEMENT.
+ * Please see the License for the specific language governing rights and
+ * limitations under the License.
+ * 
+ * @APPLE_LICENSE_HEADER_END@
+ */
+ 
+/*!
+ *  @header SLPDALocator
  */
 
 #include <stdio.h>
@@ -24,41 +42,33 @@
 #include "mslplib_opt.h" /* Definitions for optional msg support */
 #include "mslpd_store.h"
 #include "mslp_dat.h"
-//#include "mslpd.h"
 
-
-//#include "slpipc.h"
-#include "LThread.h"
 #include "SLPDALocator.h"
+#include "CNSLTimingUtils.h"
 
 static SLPDALocator* 	gDALocator = NULL;
 static pthread_mutex_t	gLock = PTHREAD_MUTEX_INITIALIZER;
 static pthread_mutex_t	gQueuedDALock = PTHREAD_MUTEX_INITIALIZER;
 static pthread_mutex_t	gTheSLPDALLock = PTHREAD_MUTEX_INITIALIZER;
-//static Boolean			gLockInitialized = false;
 
-void WakeDALocator(CFRunLoopTimerRef timer, void *info);
+#include <syslog.h>
+void StartDALocator(CFRunLoopTimerRef timer, void *info);
 
 // This is our C function wrapper to start this threaded object
-SLPInternalError StartSLPDALocator( void* daadvert_callback, SLPHandle serverState )
+SLPInternalError StartSLPDALocator( void* daadvert_callback, CFRunLoopRef runLoop, SLPHandle serverState )
 {
 	SLPInternalError	status = SLP_OK;
     SLP_LOG( SLP_LOG_DEBUG, "StartSLPDALocator called." );
-	
-/*    if (!gLockInitialized)
-    {
-        gLockInitialized = true;
-        pthread_mutex_init( &gLock, NULL );
-        
-        pthread_mutex_init( &gQueuedDALock, NULL );
-    }
-*/    
+
     LockGlobalDATable();
     status = SLPDALocator::TheSLPDAL()->Initialize( daadvert_callback, serverState );
-    
+
+	if ( runLoop )
+		SLPDALocator::TheSLPDAL()->SetRunLoop( runLoop );
+	
     if ( !status && !SLPDALocator::TheSLPDAL()->IsRunning() )
     {
-        SLP_LOG( SLP_LOG_DEBUG, "SLPDALocator isn't running yet, calling Resume" );
+        SLP_LOG( SLP_LOG_DEBUG, "SLPDALocator isn't running yet, calling Start" );
         SLPDALocator::TheSLPDAL()->Start();
     }
     else
@@ -78,12 +88,20 @@ void KickSLPDALocator( void )
 
 void StopSLPDALocator( void )
 {
+    ::pthread_mutex_lock( &gTheSLPDALLock );
     SLPDALocator* 	curDAAdvertiser = gDALocator;
     
     gDALocator = NULL;
     
-    if ( curDAAdvertiser )
-        curDAAdvertiser->Cancel();
+    if ( curDAAdvertiser && curDAAdvertiser->IsLookupInProgress() )
+	{
+		curDAAdvertiser->DeleteSelfWhenFinished();
+	}
+	else if ( curDAAdvertiser )
+	{
+		delete( curDAAdvertiser );
+	}
+    ::pthread_mutex_unlock( &gTheSLPDALLock );
 }
 
 int GlobalDATableCreationCompleted( void )
@@ -135,7 +153,7 @@ SLPDALocator* SLPDALocator::TheSLPDAL( void )
     
     if ( !gDALocator )
     {
-        SLP_LOG( SLP_LOG_DEBUG, "creating a new DA Locator thread" );
+        SLP_LOG( SLP_LOG_DEBUG, "Setting up a new DA Locator" );
         
         gDALocator = new SLPDALocator();
     }
@@ -146,7 +164,6 @@ SLPDALocator* SLPDALocator::TheSLPDAL( void )
 }
 
 SLPDALocator::SLPDALocator()
-	: LThread(threadOption_Default)
 {
     mServerState = NULL;
     mDACallback = NULL;
@@ -154,7 +171,7 @@ SLPDALocator::SLPDALocator()
     mIsRunning = false;
     mLookupInProgress = false;
     mTableReset = false;
-    mCanceled = false;
+    mDeleteSelfWhenFinished = false;
     mRunLoopRef = NULL;
     mDATableInitialized = false;
     mInitialDALookupStillPending = true;
@@ -176,16 +193,18 @@ SLPDALocator::~SLPDALocator()
     
     CLOSESOCKET(mSocket);
     mSocket = 0;
+
+	if ( mTimer )
+	{
+		if ( CFRunLoopTimerIsValid( mTimer ) )
+			CFRunLoopTimerInvalidate( mTimer );
+		CFRelease( mTimer );
+	}
+	mTimer = NULL;
     
     SLP_LOG( SLP_LOG_DEBUG, "DA Locator has been killed" );
 }
 
-void SLPDALocator::KillSLPDALocator( void )
-{
-	void* volatile		result	= NULL;
-	this->DeleteThread(result);
-}
-
 SLPInternalError SLPDALocator::Initialize( void* daadvert_callback, SLPHandle serverState )
 {
     SLP_LOG( SLP_LOG_DEBUG, "Initialize called with callback: 0x%x, serverState: 0x%x", daadvert_callback, serverState );
@@ -205,7 +224,7 @@ SLPInternalError SLPDALocator::Initialize( void )
 
     // ALERT, we must initialize our DATable first before calling SLPOpen!
     // this is because SLPOpen will then call us to get the DATable to fill into
-    // mServerState - got all that?
+    // mServerState
 	if ( !mDATableInitialized )
     {
         mDATable = dat_init();
@@ -259,15 +278,15 @@ void SLPDALocator::Kick( void )
 		int i;
 		for (i = 0; i < mDATable->iSize; i++) 
 		{
-		SLPFree(mDATable->pDAE[i].pcScopeList);
-		mDATable->pDAE[i].pcScopeList = NULL;
+			SLPFree(mDATable->pDAE[i].pcScopeList);
+			mDATable->pDAE[i].pcScopeList = NULL;
 		}
 		mDATable->iSize = 0;
 		
 		UnlockGlobalDATable();
-	
-		if ( mRunLoopRef && !mLookupInProgress )
-			CFRunLoopStop( mRunLoopRef );
+
+		CFRunLoopTimerSetNextFireDate( mTimer, CFAbsoluteTimeGetCurrent()+0 );
+
 	}
 }
 
@@ -280,7 +299,7 @@ DATable* SLPDALocator::GetDATableForRequester( void )
 {
     while ( mDALookupHasntHadAChanceToFindADAYet )
     {
-        sleep(2);
+        SmartSleep(2*USEC_PER_SEC);
         
         mDALookupHasntHadAChanceToFindADAYet = false;    
     }
@@ -288,13 +307,27 @@ DATable* SLPDALocator::GetDATableForRequester( void )
     return mDATable;
 }
 
-void SLPDALocator::Start( void )
+void SLPDALocator::SetRunLoop( CFRunLoopRef runLoop )
 {
-    mIsRunning = true;
-    this->Resume();
+	mRunLoopRef = runLoop;
 }
 
-void* SLPDALocator::Run()
+void SLPDALocator::Start( void )
+{
+    if ( mRunLoopRef )		// If we haven't been given a runloop ref yet, just ignore this call to start us.
+	{
+		mIsRunning = true;
+	
+		CFRunLoopTimerContext 		c = {0, this, NULL, NULL, NULL};
+	
+		mTimer = CFRunLoopTimerCreate(NULL, CFAbsoluteTimeGetCurrent() + 0, CONFIG_DA_FIND, 0, 0, StartDALocator, (CFRunLoopTimerContext*)&c);
+		CFRunLoopAddTimer(mRunLoopRef, mTimer, kCFRunLoopDefaultMode);
+	}
+}
+
+
+//void* SLPDALocator::Run()
+void SLPDALocator::DoLookup( void )
 {
     /*
     *  discover DAs actively
@@ -313,92 +346,89 @@ void* SLPDALocator::Run()
     long 		sleepValue = (random()*2)/LONG_MAX;
     SLPInternalError	err = SLP_OK;
     
-    mIsRunning = true;
-    mRunLoopRef = CFRunLoopGetCurrent();
+	CFRunLoopTimerSetNextFireDate( mTimer, CFAbsoluteTimeGetCurrent()+CONFIG_DA_FIND );
 
-    CFRunLoopTimerContext 	c = {0, mRunLoopRef, NULL, NULL, NULL};
+    mIsRunning = true;
+	if ( !mRunLoopRef )
+		mRunLoopRef = CFRunLoopGetCurrent();
     
-    sleep( sleepValue );
+    sleep( sleepValue*USEC_PER_SEC );
     
-    while ( !mCanceled )
-    {
-        mTableReset = false;
-        mLookupInProgress = true;
-        
-        if ( Initialize() == SLP_OK )
+	mTableReset = false;
+	
+	if ( Initialize() == SLP_OK )
+	{
+		// do this every time
+		SLP_LOG( SLP_LOG_DEBUG,"SLPDALocator starting active DA discovery");
+		
+		::pthread_mutex_lock( &gQueuedDALock );
+		if ( mNumQueuedDAsToLookup && !mTableReset )		// need to clear these up
 		{
-			// do this every time
-			SLP_LOG( SLP_LOG_DEBUG,"SLPDALocator starting active DA discovery");
-			
-			::pthread_mutex_lock( &gQueuedDALock );
-			if ( mNumQueuedDAsToLookup && !mTableReset )		// need to clear these up
+			for ( int i=0; i< mNumQueuedDAsToLookup && !mTableReset; i++ )
 			{
-				for ( int i=0; i< mNumQueuedDAsToLookup && !mTableReset; i++ )
-				{
-					LocateAndAddDA(mQueuedDAsToLookup[i]);
-				}
-				
-				mNumQueuedDAsToLookup = 0;
-				free( mQueuedDAsToLookup );
-				mQueuedDAsToLookup = NULL;
-			}
-			::pthread_mutex_unlock( &gQueuedDALock );
-			
-			char*	endPtr = NULL;
-			if ( !OnlyUsePreConfiguredDAs() && (( err = active_da_discovery(	(SLPHandle)mServerState,
-												DADISCMSEC,
-												mSocket, 
-												strtol(SLPGetProperty("net.slp.MTU"),&endPtr,10),
-												mSockAddr_in, 
-												pcScopes,
-												(void *)mDATable,
-												(void *)mDACallback,
-												SLPDAADVERT_CALLBACK) ) < 0) )
-			{
-				if (err != SLP_NETWORK_TIMED_OUT )
-				{
-					mslplog(SLP_LOG_MSG,"SLPDALocator could not do DA discovery",slperror(err));
-				}
-				else
-				{
-					SLP_LOG( SLP_LOG_DEBUG,"SLPDALocator found no DAs");
-				}
+				LocateAndAddDA(mQueuedDAsToLookup[i]);
 			}
 			
-			CLOSESOCKET(mSocket);
-			mSocket = 0;
-        }
-		else if ( mServerState )
-		{
-			SLPClose( mServerState );
-			mServerState = NULL;
+			mNumQueuedDAsToLookup = 0;
+			free( mQueuedDAsToLookup );
+			mQueuedDAsToLookup = NULL;
 		}
-		// now we sleep
-        
-        mInitialDALookupStillPending = false;
-        mDALookupHasntHadAChanceToFindADAYet = false;
-        mLookupInProgress = false;
-    
-        if ( !mTableReset )
-        {
-            CFRunLoopTimerRef 			timer = CFRunLoopTimerCreate(NULL, CFAbsoluteTimeGetCurrent() + CONFIG_DA_FIND, 0, 0, 0, WakeDALocator, (CFRunLoopTimerContext*)&c);
-            CFRunLoopAddTimer(mRunLoopRef, timer, kCFRunLoopDefaultMode);
-    
-            SLP_LOG( SLP_LOG_DEBUG, "SLPDALocator going to sleep, 0x%x\n", (void*)this );
-            CFRunLoopRun();
-            SLP_LOG( SLP_LOG_DEBUG, "SLPDALocator waking from sleep, 0x%x\n", (void*)this );
-    
-            CFRelease( timer );
-        }
-    }
-    
-	return NULL;
+		::pthread_mutex_unlock( &gQueuedDALock );
+		
+		char*	endPtr = NULL;
+
+		if ( !OnlyUsePreConfiguredDAs() && (( err = active_da_discovery(	(SLPHandle)mServerState,
+											DADISCMSEC,
+											mSocket, 
+											strtol(SLPGetProperty("net.slp.MTU"),&endPtr,10),
+											mSockAddr_in, 
+											pcScopes,
+											(void *)mDATable,
+											(void *)mDACallback,
+											SLPDAADVERT_CALLBACK) ) < 0) )
+		{
+			if (err != SLP_NETWORK_TIMED_OUT )
+			{
+				mslplog(SLP_LOG_MSG,"SLPDALocator could not do DA discovery",slperror(err));
+			}
+			else
+			{
+				SLP_LOG( SLP_LOG_DEBUG,"SLPDALocator found no DAs");
+			}
+		}
+		
+		CLOSESOCKET(mSocket);
+		mSocket = 0;
+	}
+	else if ( mServerState )
+	{
+		SLPClose( mServerState );
+		mServerState = NULL;
+	}
+	// now we sleep
+	
+	mInitialDALookupStillPending = false;
+	mDALookupHasntHadAChanceToFindADAYet = false;
+	mLookupInProgress = false;
+	
+	if ( mDeleteSelfWhenFinished )	// we were canceled but in the midst of processing
+		delete this;
 }
 
-void WakeDALocator(CFRunLoopTimerRef timer, void *info) 
+void StartDALocator(CFRunLoopTimerRef timer, void *info) 
 {
-    SLP_LOG( SLP_LOG_DEBUG, "WakeDALocator called\n" );
-    CFRunLoopStop( (CFRunLoopRef)info );
+    ::pthread_mutex_lock( &gTheSLPDALLock );
+    SLPDALocator*		daLocator = (SLPDALocator*)info;
+	
+	SLP_LOG( SLP_LOG_DEBUG, "StartDALocator timer function called\n" );
+	
+	if ( daLocator )
+		daLocator->LookupInProgress();
+		
+    ::pthread_mutex_unlock( &gTheSLPDALLock );
+
+	if ( daLocator )
+		daLocator->DoLookup();
 }
 
 void SLPDALocator::LocateAndAddDA( long addrOfDA )
@@ -425,8 +455,8 @@ void SLPDALocator::LocateAndAddDA( long addrOfDA )
     else
     {
         char*					endPtr = NULL;
-		int         			iMTU = strtol(SLPGetProperty("net.slp.MTU"),&endPtr,10);
-        int						iSize = strtol(SLPGetProperty("net.slp.MTU"),&endPtr,10);
+		int         			iMTU = (SLPGetProperty("net.slp.MTU"))?strtol(SLPGetProperty("net.slp.MTU"),&endPtr,10):1400;
+        int						iSize = iMTU;
         char					*pcRecvBuf = safe_malloc(RECVMTU,0,0);
         char					*pcSendBuf = safe_malloc(iMTU,0,0);
         int						len    = 0;   /* This records the reply length. */
@@ -473,7 +503,7 @@ void SLPDALocator::LocateAndAddDA( long addrOfDA )
                     {
                         SLPFree(pcRecvBuf);
                         pcRecvBuf = NULL;
-    //                    last_one(err, ALL_DONE,pvUser,(SLPHandle)puas,pvCallback,cbt);	  
+
                         SLP_LOG(SLP_LOG_DEBUG, "get_reply overflow, tcp failed from [%s] when locating and adding the DA...: %s",inet_ntoa(sin.sin_addr), slperror(err));
                     }
                 }
@@ -494,8 +524,8 @@ void SLPDALocator::LocateAndAddDA( long addrOfDA )
 void SLPDALocator::AskDAForScopeSponserInfo( long addrOfDA )
 {
 	char*					endPtr = NULL;
-    int         			iMTU = strtol(SLPGetProperty("net.slp.MTU"),&endPtr,10);
-    int						iSize = strtol(SLPGetProperty("net.slp.MTU"),&endPtr,10);
+    int         			iMTU = (SLPGetProperty("net.slp.MTU"))?strtol(SLPGetProperty("net.slp.MTU"),&endPtr,10):1400;
+    int						iSize = iMTU;
     char					*pcRecvBuf = safe_malloc(RECVMTU,0,0);
     char					*pcSendBuf = safe_malloc(iMTU,0,0);
     int						len    = 0;   /* This records the reply length. */
@@ -542,7 +572,7 @@ void SLPDALocator::AskDAForScopeSponserInfo( long addrOfDA )
                 {
                     SLPFree(pcRecvBuf);
                     pcRecvBuf = NULL;
-//                    last_one(err, ALL_DONE,pvUser,(SLPHandle)puas,pvCallback,cbt);	  
+
                     SLP_LOG(SLP_LOG_DEBUG, "get_reply overflow, tcp failed from [%s]...: %s",inet_ntoa(sin.sin_addr), slperror(err));
                 }
             }
@@ -558,5 +588,3 @@ void SLPDALocator::AskDAForScopeSponserInfo( long addrOfDA )
     if ( pcSendBuf )
         SLPFree(pcSendBuf);
 }
-
-
